@@ -1587,12 +1587,13 @@ do_pull (GVfsBackend *backend,
   GFile *local_file = NULL;
   GFileInfo *local_info = NULL;
   GFileInfo *info = NULL;
+  guint64 mtime;
 
   CacheEntry *entry = get_cache_entry (G_VFS_BACKEND_MTP (backend), source);
   if (entry == NULL) {
     g_vfs_job_failed_literal (G_VFS_JOB (job),
                               G_IO_ERROR, G_IO_ERROR_NOT_FOUND,
-                              _("File does not exist"));
+                              _("File doesn't exist"));
     goto exit;
   } else if (entry->id == -1) {
     g_vfs_job_failed_literal (G_VFS_JOB (job),
@@ -1608,7 +1609,7 @@ do_pull (GVfsBackend *backend,
   if (file == NULL) {
     g_vfs_job_failed_literal (G_VFS_JOB (job),
                               G_IO_ERROR, G_IO_ERROR_NOT_FOUND,
-                              _("File does not exist"));
+                              _("File doesn't exist"));
     goto exit;
   }
 
@@ -1693,6 +1694,19 @@ do_pull (GVfsBackend *backend,
     fail_job (G_VFS_JOB (job), device);
     goto exit;
   }
+
+  /* Ignore errors here. Failure to copy metadata is not a hard error */
+  mtime = g_file_info_get_attribute_uint64 (info,
+                                            G_FILE_ATTRIBUTE_TIME_MODIFIED);
+  g_file_set_attribute_uint64 (local_file,
+                               G_FILE_ATTRIBUTE_TIME_MODIFIED, mtime,
+                               G_FILE_QUERY_INFO_NONE,
+                               G_VFS_JOB (job)->cancellable, NULL);
+  g_file_set_attribute_uint32 (local_file,
+                               G_FILE_ATTRIBUTE_TIME_MODIFIED_USEC, 0,
+                               G_FILE_QUERY_INFO_NONE,
+                               G_VFS_JOB (job)->cancellable, NULL);
+
   /* Attempt to delete object if requested but don't fail it it fails. */
   if (remove_source) {
     DEBUG ("(I) Removing source.\n");
@@ -2049,7 +2063,7 @@ do_delete (GVfsBackend *backend,
   if (entry == NULL) {
     g_vfs_job_failed_literal (G_VFS_JOB (job),
                               G_IO_ERROR, G_IO_ERROR_NOT_FOUND,
-                              _("File does not exist"));
+                              _("File doesn't exist"));
     goto exit;
   } else if (entry->id == -1) {
     g_vfs_job_failed_literal (G_VFS_JOB (job),
@@ -2060,6 +2074,26 @@ do_delete (GVfsBackend *backend,
 
   LIBMTP_mtpdevice_t *device;
   device = G_VFS_BACKEND_MTP (backend)->device;
+  LIBMTP_file_t *file = LIBMTP_Get_Filemetadata (device, entry->id);
+  if (file->filetype == LIBMTP_FILETYPE_FOLDER) {
+    LIBMTP_file_t *files;
+    LIBMTP_Clear_Errorstack (device);
+    files = LIBMTP_Get_Files_And_Folders (device, entry->storage, entry->id);
+    if (LIBMTP_Get_Errorstack (device) != NULL) {
+      g_vfs_job_failed_literal (G_VFS_JOB (job),
+                                G_IO_ERROR, G_IO_ERROR_NOT_EMPTY,
+                                _("Error checking for directory emptiness"));
+      goto exit;
+    }
+
+    if (files != NULL) {
+      g_vfs_job_failed_literal (G_VFS_JOB (job),
+                                G_IO_ERROR, G_IO_ERROR_NOT_EMPTY,
+                                g_strerror (ENOTEMPTY));
+      DEBUG ("(II) Directory size %d\n", file->filesize);
+      goto exit;
+    }
+  }
 
   int ret = LIBMTP_Delete_Object (device, entry->id);
   if (ret != 0) {
@@ -2094,7 +2128,7 @@ do_set_display_name (GVfsBackend *backend,
   if (entry == NULL) {
     g_vfs_job_failed_literal (G_VFS_JOB (job),
                               G_IO_ERROR, G_IO_ERROR_NOT_FOUND,
-                              _("File does not exist"));
+                              _("File doesn't exist"));
     goto exit;
   } else if (entry->id == -1) {
     g_vfs_job_failed_literal (G_VFS_JOB (job),
@@ -2161,7 +2195,7 @@ do_open_for_read (GVfsBackend *backend,
   if (entry == NULL) {
     g_vfs_job_failed_literal (G_VFS_JOB (job),
                               G_IO_ERROR, G_IO_ERROR_NOT_FOUND,
-                              _("File does not exist"));
+                              _("File doesn't exist"));
     goto exit;
   } else if (entry->id == -1) {
     g_vfs_job_failed_literal (G_VFS_JOB (job),
@@ -2507,7 +2541,7 @@ do_append_to (GVfsBackend *backend,
   if (entry == NULL) {
     g_vfs_job_failed_literal (G_VFS_JOB (job),
                               G_IO_ERROR, G_IO_ERROR_NOT_FOUND,
-                              _("File does not exist"));
+                              _("File doesn't exist"));
     goto exit;
   } else if (entry->id == -1) {
     g_vfs_job_failed_literal (G_VFS_JOB (job),
