@@ -195,7 +195,8 @@ list (GFile *file)
 
   return res;
 }
-
+static char*
+shell_quote (const gchar *unquoted_string);
 static void
 print_mounts (const char *prefix)
 {
@@ -218,7 +219,7 @@ print_mounts (const char *prefix)
 	  uri = g_file_get_uri (mount_root);
 	  if (prefix == NULL ||
 	      g_str_has_prefix (uri, prefix))
-	    g_print ("%s\n", uri);
+	    g_print ("%s%s\n", shell_quote (uri), g_str_has_suffix (uri, "/") ? "" : "/");
 	  g_free (uri);
 	  g_object_unref (mount_root);
 	  g_object_unref (mount);
@@ -253,6 +254,8 @@ shell_quote (const gchar *unquoted_string)
 	g_string_append (dest, "\\'");
       else if (*p == '"')
 	g_string_append (dest, "\\\"");
+      else if (*p == ':')
+	g_string_append (dest, "\\:");
       else
 	g_string_append_c (dest, *p);
 
@@ -271,7 +274,7 @@ show_completed_file (GFile *hit,
   GFile *cwd_f;
   GFile *home;
 
-  if (g_file_is_native (hit))
+  if (g_file_is_native (hit) && !g_str_has_prefix (g_shell_unquote(arg, NULL), "file://"))
     {
       cwd = g_get_current_dir ();
       cwd_f = g_file_new_for_path (cwd);
@@ -302,7 +305,7 @@ show_completed_file (GFile *hit,
   else
     display = g_file_get_uri (hit);
 
-  g_print ("%s%s\n", display, (is_dir)?"/":"");
+  g_print ("%s%s\n", shell_quote (display), (is_dir)?"/":"");
   g_free (display);
 }
 
@@ -342,12 +345,7 @@ print_completions (const char *arg)
       strchr (arg, '/') == NULL ||
       !g_file_query_exists (parent, NULL))
     {
-      GMount *mount;
-      mount = g_file_find_enclosing_mount (f, NULL, NULL);
-      if (mount == NULL)
-	print_mounts (unescaped);
-      else
-	g_object_unref (mount);
+      print_mounts (unescaped);
     }
 
   if (parent != NULL)
